import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:camera/camera.dart';
import 'dart:io';
import 'package:wechat_assets_picker/wechat_assets_picker.dart';
import 'listCameraFlashMode.dart';

CameraController? _cameraController;

class OpenCamera extends StatefulWidget {
final int totalQuestion;
final bool switchCamera;
  OpenCamera(
     {
    Key? key,this.totalQuestion = 1,this.switchCamera = false
  }) : super(key: key);

  @override
  _OpenCameraState createState() => _OpenCameraState();
}

class _OpenCameraState extends State<OpenCamera> {
  List<XFile> listImageFile = [];
  late File file;
  late int cameraType;
  late int flashMode;
  List<CameraDescription>? cameras;
  ListCameraFlashMode _listCameraFlashMode = ListCameraFlashMode();

  @override
  void initState() {

    super.initState();
    cameraType = 0;
    flashMode = 0;
    getCamera();

  }
  void getCamera()async{
    cameras = await availableCameras();
    setState(() {

    });
  }
  @override
  Widget build(BuildContext context) {
    if(cameras == null){
      return Container();
    }
    return WillPopScope(
      onWillPop: () async {
        Navigator.pop(context, listImageFile);
        return true;
      },
      child: Scaffold(
        body: Stack(
          children: [
            CameraApp(cameras![0], UniqueKey),
            Positioned(
              bottom: 0,
              child: Container(
                height: 100,
                width: MediaQuery.of(context).size.width,
                color: Color.fromRGBO(0, 0, 0, .5),
                child: Builder(builder: (context){
                  return Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: [
                      doneClick(),
                      GestureDetector(onTap: () async {
                        if(widget.totalQuestion-listImageFile.length ==0){
                          Navigator.pop(context, listImageFile);
                          return;
                        }
                        final List<AssetEntity>? receivedImage = await AssetPicker.pickAssets(context,
                          maxAssets: widget.totalQuestion-listImageFile.length,
                          textDelegate: EnglishTextDelegate(),
                        );

                        for(var i =0;receivedImage == null?false:i<receivedImage.length;i++){
                          this.listImageFile.add(XFile((await receivedImage[i].file)!.path));
                        }
                        Navigator.pop(context, listImageFile);

                      },child: Container(
                        height:40,
                        width:40,
                        child: ClipRRect(

                  borderRadius:BorderRadius.all(Radius.circular(20)),

                            child: listImageFile.length>0?Image.file(File(listImageFile[listImageFile.length-1].path),fit:BoxFit.fill):Container(color: Colors.blue,)),
                      )),
                      cameraControl(),
                      switchCamera(),
                      flashButtonSwitch(),
                    ],
                  );
                } ,
                ),
              ),
            ),

          ],
        ),
      ),
    );
  }

  GestureDetector flashButtonSwitch() {
    return GestureDetector(
                          onTap: () {

                            if (flashMode == _listCameraFlashMode.listCameraFlashMode.length - 1) {
                              flashMode = 0;
                            } else {
                              flashMode = flashMode + 1;
                            }
                            _cameraController!.setFlashMode(_listCameraFlashMode.listCameraFlashMode[flashMode].flashMode);

                            setState(() {});
                          },
                          child: Icon(
                              _listCameraFlashMode.listCameraFlashMode[flashMode].iconData,
                              color: Colors.white));
  }

  GestureDetector switchCamera() {
    return GestureDetector(
        onTap: () {
          cameraType = cameraType == 0 ? 1 : 0;
          _cameraController = CameraController(
              cameras![cameraType], ResolutionPreset.max);
          _cameraController!.initialize().then((_) {
            setState(() {});
          });
        },
        child: Icon(Icons.switch_camera,color: Colors.white,));
  }

  GestureDetector cameraControl() {

    return GestureDetector(
        onTap: () {
          onTakePictureButtonPressed();
        },
        child: Icon(
          Icons.camera,
          color: Colors.white,
          size: 60,
        ));
  }

  GestureDetector doneClick(){
    return GestureDetector(
      onTap: (){
        Navigator.pop(context, listImageFile);
      },
      child: ClipRRect(
        borderRadius: BorderRadius.all(Radius.circular(20)),
        child: Container(
          height: 40,
          color: Colors.white,
        child: Center(
          child: Text(listImageFile.length.toString(),style: TextStyle(fontSize: 20),),
        ),),
      ),
    );
  }
  void onTakePictureButtonPressed() async {
    Future<XFile?> takePicture() async {
      final CameraController? cameraController = _cameraController;
      if (cameraController == null || !cameraController.value.isInitialized) {
        return null;
      }

      if (cameraController.value.isTakingPicture) {
        // A capture is already pending, do nothing.
        return null;
      }

      try {
        XFile file = await cameraController.takePicture();

        return file;
      } on CameraException catch (e) {
        return null;
      }
    }

    takePicture().then((XFile? file) {
      if (file != null) {
        if(widget.totalQuestion ==listImageFile.length){
          listImageFile.removeAt(0);
        }
        listImageFile.add(file);
      }
      setState(() {});
    });
  }
}

class CameraApp extends StatefulWidget {
  CameraDescription cameras;

  CameraApp(this.cameras, key);

  @override
  _CameraAppState createState() => _CameraAppState();
}

class _CameraAppState extends State<CameraApp> {
  @override
  void initState() {
    super.initState();
    _cameraController = CameraController(widget.cameras, ResolutionPreset.max);
    _cameraController!.initialize().then((_) {
      if (!mounted) {
        return;
      }
      setState(() {});
    });
  }

  @override
  void dispose() {
    _cameraController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (!_cameraController!.value.isInitialized) {
      return Container();
    }
    return MaterialApp(
      home: CameraPreview(_cameraController!),
    );
  }
}

class EnglishTextDelegate implements AssetsPickerTextDelegate {
  factory EnglishTextDelegate() => _instance;

  EnglishTextDelegate._internal();

  static final EnglishTextDelegate _instance = EnglishTextDelegate._internal();

  @override
  String confirm = 'Confirm';

  @override
  String cancel = 'Cancel';

  @override
  String edit = 'Edit';

  @override
  String gifIndicator = 'GIF';

  @override
  String heicNotSupported = 'Unsupported HEIC asset type.';

  @override
  String loadFailed = 'Load failed';

  @override
  String original = 'Origin';

  @override
  String preview = 'Preview';

  @override
  String select = 'Select';

  @override
  String unSupportedAssetType = 'Unsupported HEIC asset type.';

  @override
  String durationIndicatorBuilder(Duration duration) =>
      AssetsPickerTextDelegate.defaultDurationIndicatorBuilder(duration);
}